using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;

using GameEngine;

namespace ConnectFour.WCF {
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	public class GameEngineService : IGameEngineServerContract {
		public GameEngineService( ) {
		}
		
		public GameEngineService(GameEngine.Grid gameEngineGrid) {
			this.gameEngineGrid = gameEngineGrid;
		}

		GameEngine.Grid gameEngineGrid;
		public GameEngine.Grid GameEngineGrid {
			get { return gameEngineGrid; }
			set { gameEngineGrid = value; }
		}

		public static GameEngineService Publish(Grid grid, string baseURI, string serviceName) {
			GameEngineService service = new GameEngineService(grid);
			ServiceHost serviceHost = new ServiceHost(service, new Uri(baseURI));
			Binding binding = new NetTcpBinding(SecurityMode.None);
			serviceHost.AddServiceEndpoint(typeof(IGameEngineServerContract), binding, serviceName);
			serviceHost.Open( );
			return service;
		}

		public static IGameEngineServerContract GetGameEngine(string connectionString, out GameEngineService ownGameEngineService) {
			Binding binding = new NetTcpBinding(SecurityMode.None);
			ownGameEngineService = new GameEngineService( );
			DuplexChannelFactory<IGameEngineServerContract> factory =
				new DuplexChannelFactory<IGameEngineServerContract>(ownGameEngineService, binding, connectionString);
			return factory.CreateChannel( );
		}

		public event EventHandler<RemoteGameEngineEventArgs> ReceivedRemoteGameEngine;
		protected virtual void OnReceivedRemoteGameEngine(IGameEngineContract engine) {
			if (ReceivedRemoteGameEngine != null)
				ReceivedRemoteGameEngine(this, new RemoteGameEngineEventArgs(engine));
		}
        
		#region IGameEngineContract Members

		DiscType IGameEngineContract.GetNextPlayer( ) {
			IGameEngineContract remoteContract = OperationContext.Current.GetCallbackChannel<IGameEngineContract>( );
			if (remoteContract != null)
				OnReceivedRemoteGameEngine(remoteContract);
			return gameEngineGrid.NextPlayer;
		}

		void IGameEngineContract.AcceptMove(DiscType player, int column) {
			gameEngineGrid.AcceptMove(player, column);
		}

		#endregion
	}
}