using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.Specialized;

using GameEngine;
using ConnectFour.WCF;


namespace ConnectFour.WPF {
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>

	public partial class ConnectFourMainWindow : System.Windows.Window {
		public ConnectFourMainWindow( ) {
			InitializeComponent( );
		}

		private bool CheckInitialization( ) {
			if (App.ClientRole == ClientRole.None) {
				MessageBox.Show("You must start the application as a Server or a Client first.");
				return false;
			}
			if (App.ClientRole == ClientRole.Server && gameEngineService == null) {
				MessageBox.Show("There is no client connected so far.");
				return false;
			}
			return true;
		}

		private void OnGridColumnMouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
			if (!CheckInitialization( ))
				return;

			if (grid.Winner != null) {
				MessageBox.Show("The game is over.");
				return;
			}
			if (gameEngineService.GetNextPlayer( ) != myDiscType) {
				MessageBox.Show("It's not your turn to play.");
				return;
			}

			ItemsControl itemsControl = sender as ItemsControl;
			if (itemsControl != null) {
				Column clickedColumn = itemsControl.Tag as Column;
				int colIndex = grid.Columns.IndexOf(clickedColumn);
				grid.AcceptMove(myDiscType, colIndex);
				gameEngineService.AcceptMove(myDiscType, colIndex);
			}
		}

		// Our own grid - we want our own instance, if only for easy data binding
		GameEngine.Grid grid;
		// An interface to the grid on the other side
		IGameEngineContract gameEngineService;
		// My disc type - hard coded, red for the server, yellow for the client
		DiscType myDiscType;

		private void OnStartServer( object sender, RoutedEventArgs e) {
			grid = new GameEngine.Grid( );
			GameEngineService publishedService = GameEngineService.Publish(
				grid, ServerBaseURITextBox.Text, ServiceNameTextBox.Text);
			publishedService.ReceivedRemoteGameEngine += new EventHandler<RemoteGameEngineEventArgs>(publishedService_ReceivedRemoteGameEngine);
			myDiscType = DiscType.Red;
			App.ClientRole = ClientRole.Server;
			ConnectClientButton.IsEnabled = StartServerButton.IsEnabled = false;
			DataContext = grid;
		}

		void publishedService_ReceivedRemoteGameEngine(object sender, RemoteGameEngineEventArgs e) {
			gameEngineService = e.RemoteGameEngine;			
		}

		private void OnConnectClient(object sender, RoutedEventArgs e) {
			GameEngineService ownGameEngineService;
			gameEngineService = GameEngineService.GetGameEngine(ClientConnectionTextBox.Text, out ownGameEngineService);
			grid = new GameEngine.Grid(gameEngineService.GetNextPlayer( ));
			ownGameEngineService.GameEngineGrid = grid;
			myDiscType = DiscType.Yellow;
			App.ClientRole = ClientRole.Client;
			ConnectClientButton.IsEnabled = StartServerButton.IsEnabled = false;
			DataContext = grid;
		}
	}
}