using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

namespace GameEngine.Tests {
	[TestFixture]
	public class GridTests {
		Grid grid;
		DiscType startingPlayer;
		DiscType oppositePlayer;

		[SetUp]
		public void Setup( ) {
			grid = new Grid( );
			startingPlayer = grid.NextPlayer;
			oppositePlayer = startingPlayer == DiscType.Red ? DiscType.Yellow : DiscType.Red;
		}

		[Test]
		public void Construction( ) {
			Assert.IsNotNull(grid.Columns);
			Assert.AreEqual(7, grid.Columns.Count);
			Assert.AreEqual(4, grid.SequenceLengthForWin);
			Assert.IsNull(grid.Winner);
		}

		[Test]
		public void GameFlowPlayerChange( ) {
			grid.AcceptMove(startingPlayer, 0);
			Assert.AreEqual(oppositePlayer, grid.NextPlayer);
			grid.AcceptMove(oppositePlayer, 0);
			Assert.AreEqual(startingPlayer, grid.NextPlayer);
			grid.AcceptMove(startingPlayer, 0);
			Assert.AreEqual(oppositePlayer, grid.NextPlayer);
		}

		[Test]
		[ExpectedException(typeof(InvalidOperationException), "Wrong player.")]
		public void GameFlowWrongPlayer( ) {
			grid.AcceptMove(oppositePlayer, 0);
		}

		[Test]
		public void ValidSequenceLength( ) {
			grid.SequenceLengthForWin = 2;
			Assert.AreEqual(2, grid.SequenceLengthForWin);
			grid.SequenceLengthForWin = 3;
			Assert.AreEqual(3, grid.SequenceLengthForWin);
			grid.SequenceLengthForWin = 4;
			Assert.AreEqual(4, grid.SequenceLengthForWin);
			grid.SequenceLengthForWin = 5;
			Assert.AreEqual(5, grid.SequenceLengthForWin);
			grid.SequenceLengthForWin = 6;
			Assert.AreEqual(6, grid.SequenceLengthForWin);
			grid.SequenceLengthForWin = 7;
			Assert.AreEqual(7, grid.SequenceLengthForWin);
		}

		[Test]
		[ExpectedException(typeof(ArgumentException), "SequenceLengthForWin can't be greater than Min(RowCount, 7).")]
		public void InvalidSequenceLength( ) {
			grid.RowCount = 5;
			grid.SequenceLengthForWin = 6;
		}
	}
}
