using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

namespace GameEngine.Tests {
	[TestFixture]
	public class WinnerTests {
		Grid grid;
		DiscType startingPlayer;
		DiscType oppositePlayer;

		[SetUp]
		public void Setup( ) {
			grid = new Grid( );
			startingPlayer = grid.NextPlayer;
			oppositePlayer = startingPlayer == DiscType.Red ? DiscType.Yellow : DiscType.Red;
		}

		private void MakeMoveAndCheckForNullWinner(DiscType player, int column) {
			grid.AcceptMove(player, column);
			Assert.IsNull(grid.Winner);
		}

		[Test]
		public void VerticalWinner1( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			grid.AcceptMove(startingPlayer, 0);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void VerticalWinner2( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 3);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 4);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 6);
			grid.AcceptMove(startingPlayer, 3);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void HorizontalWinner1( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 0);
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			grid.AcceptMove(startingPlayer, 3);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void HorizontalWinner2( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 4);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 5);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			grid.AcceptMove(startingPlayer, 6);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void HorizontalNonWinner( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 3);
			MakeMoveAndCheckForNullWinner(startingPlayer, 4);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 4);
			MakeMoveAndCheckForNullWinner(startingPlayer, 5);
		}

		[Test]
		public void DiagonalWinner1( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 3);
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 3);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 0);
			grid.AcceptMove(startingPlayer, 3);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void DiagonalWinner2( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 0);
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 0);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 4);
			grid.AcceptMove(startingPlayer, 3);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}

		[Test]
		public void DiagonalWinner3( ) {
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 1);
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			MakeMoveAndCheckForNullWinner(startingPlayer, 1);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 2);
			MakeMoveAndCheckForNullWinner(startingPlayer, 2);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 3);
			MakeMoveAndCheckForNullWinner(startingPlayer, 3);
			MakeMoveAndCheckForNullWinner(oppositePlayer, 5);
			grid.AcceptMove(startingPlayer, 4);
			Assert.AreEqual(startingPlayer, grid.Winner);
		}
	}
}
